/* Reverse polish notation calculator.  */
     
     %{
       #define YYSTYPE node*
       #include <math.h>
       #include <stdio.h>
       #include <ctype.h>
       #include "lex.yy.c"
       void yyerror (char const *);
     %}
     
/* Bison declarations.  */
     %token NUM
     %left '-' '+'
     %left '*' '/'
     %left NEG     /* negation--unary minus */
     %right '^'    /* exponentiation */
     
     %% /* The grammar follows.  */
     input:    /* empty */
             | input line
     ;
     
     line:     '\n'
             | exp '\n'   { $2 = conNode("R");
                            $$ = conTree2("line", $1, $2);    
                            traverseTree($$, 1); }
             | error '\n' { yyerrok;                  }
     ;
     
     exp:      NUM                { $$ = $1;         }
             | exp '+' exp        { $2 = conNode("+");
                                    $$ = conTree3("exp", $1, $2, $3);    }
             | exp '-' exp        { $2 = conNode("-");
                                    $$ = conTree3("exp", $1, $2, $3);    }
             | exp '*' exp        { $2 = conNode("*");
                                    $$ = conTree3("exp", $1, $2, $3);    }
             | exp '/' exp        { $2 = conNode("/");
                                    $$ = conTree3("exp", $1, $2, $3);    }
             | '-' exp  %prec NEG { $1 = conNode("-");
                                    $$ = conTree2("exp", $1, $2);    }
             | exp '^' exp        { $2 = conNode("^");
                                    $$ = conTree3("exp", $1, $2, $3);    }
             | '(' exp ')'        { $1 = conNode("(");
                                    $3 = conNode(")");
                                    $$ = conTree3("exp", $1, $2, $3);    }
     ;
     %%

 /* The lexical analyzer returns a double floating point
        number on the stack and the token NUM, or the numeric code
        of the character read if not a number.  It skips all blanks
        and tabs, and returns 0 for end-of-input.  */
     

int main (void)
     {
	char fname[40];
	char ch;

	/* Read in the filename */
	printf("Enter the file name for parsing: ");
	scanf("%s",fname);

	/* Open the file.  If NULL is returned there was an error */
	if((stdin = fopen(fname, "r")) == 0) {
		printf("Error Opening File.\n");
		return 1;
	}

	yyparse ();
	fclose(stdin);  /* Close the file */
        return 0; 
     }

     /* Called by yyparse on error.  */
void yyerror (char const *s)
     {
       fprintf (stderr, "%s\n", s);
     }